/* $Id: arp.c,v 1.1 2000/04/04 22:22:40 bind Exp $ */

#include "sentinel.h"

int sendarp(struct libnet_link_int *);
int processarp(char *);

struct	arp_header {
	u_int16_t ar_hrd;	/* format of hardware address */
	u_int16_t ar_pro;	/* format of protocol address */
	u_int8_t  ar_hln;	/* length of hardware address */
	u_int8_t  ar_pln;	/* length of protocol address */
	u_int16_t ar_op;	/* one of: */
	u_int8_t  ar_sha[6];	/* sender hardware address */
	u_int8_t  ar_spa[6];	/* sender protocol address */
	u_int8_t  ar_tha[6];	/* target hardware address */
	u_int8_t  ar_tpa[6];	/* target protocol address */
};

int arptest(void) 
{
  pcap_t *socket;
  char *pkt;
  struct bpf_program prog;
  struct pcap_pkthdr pcap_h;
  struct libnet_link_int *link;

  unsigned int network, netmask;

  link = libnet_open_link_interface(device,errbuf);

  if(!link) {
    perror("libnet_open_link_interface");
    exit(-1);
  }

  pcap_lookupnet(device, &network, &netmask, errbuf);

  socket = pcap_open_live(device, 128, 0, 1024, errbuf);
  if(socket == NULL) {
    perror("pcap_open_live");
    exit(1);
  }
 
  pcap_compile(socket, &prog, "arp", 0, netmask);

  if(sendarp(link) < 0) {
    printf("There were errors while completing this test..\n");
    return -1;
  }
  
  pcap_setfilter(socket, &prog);
  pkt = (u_char *)pcap_next(socket, &pcap_h);

  if(pkt != NULL)
    if(processarp(pkt) == 1) { 
      pcap_close(socket); 
      return 1; 
    }
 
  pcap_close(socket);
  libnet_close_link_interface(link);
  return 0;
}

int sendarp(struct libnet_link_int *link)
{
  u_char *pkt;
  int n;

  extern u_char fakeaddr[6];
  u_char defaddr[6] = {0,0,0,0,0};

  if(libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &pkt) < 0) {
    perror("libnet_init_packet");
    exit(1);
  }

  libnet_build_ethernet(fakeaddr,fakeaddr,ETHERTYPE_ARP,NULL,0,pkt);
  
  libnet_build_arp(ARPHRD_ETHER,ETHERTYPE_IP, 6, 4, ARPOP_REQUEST,
                    (u_char *)hwsource, (u_char *)&source, defaddr, (u_char *)&dest, NULL, 0,
                      pkt + ETH_H); 

  n = libnet_write_link_layer(link, device, pkt, LIBNET_ARP_H + LIBNET_ETH_H);

  libnet_destroy_packet(&pkt);
  return n;
}  

int processarp(char *pkt)
{
  struct arp_header *arp;
 
  arp = (struct arp_header *)(pkt + ETHHDRSIZE);

  if(htons(arp->ar_op) == 2) 
    if(strstr(arp->ar_spa,(char *)&dest) != NULL) 
      return 1;

  return 0;
}

